#pragma once

#include <stdint.h>
#include <malloc.h>
#include "../../engine/display/primitive.h"
#include "../../engine/display/texture.h"
#include "../../engine/display/tileset.h"


/** Bitwise Flags from tiles. */
typedef uint8_t tileflag_t;


/** Tile ID */
typedef uint8_t tileid_t;


/** Representation of a unique tile within a chunk. */
typedef struct {
  /** ID of the tile */
  tileid_t id;

  /** Rendering indice and vertice offsets for the tile. */
  int32_t indiceStart, verticeStart;
} tile_t;


/** Representation of the information of a tile within a tilemap. */
typedef struct {
  /** Flags of the tile */
  tileflag_t flags;

  /** How many indices and vertices a tile with this definition has. */
  int32_t indiceCount, verticeCount;
} tiledef_t;


/** Representation of the tile definitions within a tilemap. */
typedef struct {
  /** Tile definitions within the tilemap */
  tiledef_t *tileDefinitions;
} tilemap_t;


/** Representation of a chunk, a group of tiles that can be buffered around. */
typedef struct chunk_t {
  /** Position (in absolute chunk coordinates) of this chunk */
  int32_t x, y, z;

  /** Array of tiles within the chunk */
  tile_t *tiles;

  /** Ready to be rendered chunk 3d primitive */
  primitive_t *primitive;
} chunk_t;


/** Representation of the world. */
typedef struct {
  /** Asset subdir name */
  char *assetWorldDirectory;

  /** Tileset texture */
  texture_t *texture;

  /** Tileset predivided */
  tileset_t *tileset;

  /** Tilemap of the world */
  tilemap_t *tilemap;

  /** Current (chunk) coordinates of the first chunk in the chunk list */
  int32_t x, y, z;

  /** Count of chunks within the chunk list */
  int32_t count;

  /** Count of tiles within each chunk */
  int32_t tileCount;

  /** Current chunk list, ordered */
  chunk_t **chunkList;

  /** Chunk array (unordered) */
  chunk_t *chunks;
} world_t;